--------------------------------------------------------------------------------
-- copyright notice:
--------------------------------------------------------------------------------
-- copyright (c) 2007 by
--
-- CESYS GmbH / GERMANY
--
-- http://www.cesys.com
--------------------------------------------------------------------------------
-- author:
--------------------------------------------------------------------------------
-- Stefan Fischer <stefan.fischer@cesys.com>
--------------------------------------------------------------------------------
-- license:
--------------------------------------------------------------------------------
-- THIS SOURCECODE IS NOT FREE! IT IS FOR USE TOGETHER WITH THE CESYS EFM01
-- USB CARD (ARTICLE-NR.: C1050-4107) ONLY! YOU ARE NOT ALLOWED TO MODIFY AND
-- DISTRIBUTE OR USE IT WITH ANY OTHER HARDWARE, SOFTWARE OR ANY OTHER KIND OF
-- ASIC OR PROGRAMMABLE LOGIC DESIGN WITHOUT THE EXPLICIT PERMISSION OF THE
-- COPYRIGHT HOLDER!
--------------------------------------------------------------------------------
-- disclaimer of warranty:
--------------------------------------------------------------------------------
-- THIS SOURCECODE IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
-- THERE IS NO WARRANTY OR SUPPORT FOR THIS SOURCECODE. THE COPYRIGHT HOLDER 
-- PROVIDES THIS SOURCECODE "AS IS" WITHOUT WARRANTY OF ANY KIND,
-- EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
-- WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
-- THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THIS SOURCECODE IS WITH
-- YOU. SHOULD THIS SOURCECODE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL 
-- NECESSARY SERVICING, REPAIR OR CORRECTION.
--
-- IN NO EVENT WILL THE COPYRIGHT HOLDER BE LIABLE TO YOU FOR DAMAGES, INCLUDING
-- ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
-- USE OR INABILITY TO USE THIS SOURCECODE (INCLUDING BUT NOT LIMITED TO LOSS OF
-- DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD 
-- PARTIES OR A FAILURE OF THIS SOURCECODE TO OPERATE WITH ANY OTHER
-- SOFTWARE-PROGRAMS, HARDWARE-CIRCUITS OR ANY OTHER KIND OF ASIC OR
-- PROGRAMMABLE LOGIC DESIGN), EVEN IF THE COPYRIGHT HOLDER HAS BEEN ADVISED OF 
-- THE POSSIBILITY OF SUCH DAMAGES.
--------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.all;

use work.wishbone.all;


entity efm01_top is
  generic
  (
    nr_of_dbgports : positive := 1
  );
  port
  (
    pin_sys_reset_n_i : in std_logic;
    
    pin_fx2_ifclk_i : in std_logic;
    pin_fx2_flag_n_i : in std_logic_vector(2 downto 0);
    pin_fx2_sloe_n_o : out std_logic;
    pin_fx2_slrd_n_o : out std_logic;
    pin_fx2_slwr_n_o : out std_logic;
    pin_fx2_pktend_n_o : out std_logic;
    pin_fx2_fifoadr_o : out std_logic_vector(1 downto 0);
    pin_fx2_fd_io : inout std_logic_vector(15 downto 0);

    pin_gpio_io : inout std_logic_vector(49 downto 0);

    pin_flash_s_n_o : out std_logic;
    pin_flash_c_o : out std_logic;
    pin_flash_d_o : out std_logic;
    pin_flash_q_i : in std_logic;

    pin_debug_io : inout std_logic_vector((nr_of_dbgports-1) downto 0)
  );
end efm01_top;


architecture RTL of efm01_top is

  signal sys_reset_n : std_logic := '0';
  
  signal fx2_ifclk : std_logic := '0';
  signal fx2_flag_n : std_logic_vector(2 downto 0) := (others => '1');
  signal fx2_sloe_n : std_logic := '1';
  signal fx2_slrd_n : std_logic := '1';
  signal fx2_slwr_n : std_logic := '1';
  signal fx2_pktend_n : std_logic := '1';
  signal fx2_fifoadr : std_logic_vector(1 downto 0) := (others => '0');
  
  signal flash_s_n : std_logic := '1';
  signal flash_c : std_logic := '0';
  signal flash_d : std_logic := '0';
  signal flash_q : std_logic := '0';
  
begin

  -- connect ports to internal signals
  ------------------------------------
  
  -- ports of system control
  sys_reset_n <= pin_sys_reset_n_i;

  -- ports of slave fifo interface
  fx2_ifclk <= pin_fx2_ifclk_i;
  fx2_flag_n <= pin_fx2_flag_n_i;
  pin_fx2_sloe_n_o <= fx2_sloe_n;
  pin_fx2_slrd_n_o <= fx2_slrd_n;
  pin_fx2_slwr_n_o <= fx2_slwr_n;
  pin_fx2_pktend_n_o <= fx2_pktend_n;
  pin_fx2_fifoadr_o <= fx2_fifoadr;
  
  -- ports of flash interface
  pin_flash_s_n_o <= flash_s_n;
  -- caution!
  -- pin_flash_c_o is driven by an external oscillator, if pin_sys_reset_n_i is low
  -- => output drivers of pin_flash_c_o must be turned off to high impedance state
  pin_flash_c_o <= flash_c when (sys_reset_n = '1') else 'Z';
  pin_flash_d_o <= flash_d;
  flash_q <= pin_flash_q_i;

  -- debug ports
  pin_debug_io <=
  (
    others => 'Z'
  );

  -- instantiate wishbone modules
  -------------------------------

  inst_wb_intercon : wb_intercon
  generic map
  (
    nr_of_dbgports => 1
  )
  port map
  (
    SYSCON_I => intercon.syscon,

    SYSCON_MA_O => intercon.masters.syscon,
    MASTER_I => intercon.masters.master,
    SLAVE_O => intercon.masters.slave,

    SYSCON_SL_O => intercon.slaves.syscon,
    MASTER_O => intercon.slaves.master,
    SLAVE_I => intercon.slaves.slave,

    debug => open
  );

  inst_wb_syscon : wb_syscon
  generic map
  (
    nr_of_dbgports => 1
  )
  port map
  (
    SYSCON_O => intercon.syscon,

    sys_rst_i => sys_rst,
    sys_clk_i => sys_clk,

    debug => open
  );
  
  sys_rst <= not sys_reset_n;
  sys_clk <= fx2_ifclk; -- whole system is fully synchronous to fx2 controller slave fifo interface
  
  inst_wb_ma_fx2 : wb_ma_fx2
  generic map
  (
    nr_of_irqiports => nr_of_irqs,
    nr_of_irqoports => nr_of_irqs,
    nr_of_dbgports => 1
  )
  port map
  (
    SYSCON_I => intercon.masters.syscon(ma_fx2_id),
    MASTER_O => intercon.masters.master(ma_fx2_id),
    SLAVE_I => intercon.masters.slave(ma_fx2_id),

    IRQ_I => irqs,
    IRQ_O => irqos(irqdev_fx2_id),

    fx2_ifclk_i => fx2_ifclk,
    fx2_flag_n_i => fx2_flag_n,
    fx2_sloe_n_o => fx2_sloe_n,
    fx2_slrd_n_o => fx2_slrd_n,
    fx2_slwr_n_o => fx2_slwr_n,
    fx2_pktend_n_o => fx2_pktend_n,
    fx2_fifoadr_o => fx2_fifoadr,
    fx2_fd_io => pin_fx2_fd_io,

    debug => open
  );

  inst_wb_sl_flash : wb_sl_flash
  generic map
  (
    nr_of_irqiports => nr_of_irqs,
    nr_of_irqoports => nr_of_irqs,
    nr_of_dbgports => 1
  )
  port map
  (
    SYSCON_I => intercon.slaves.syscon(sl_flash_id),
    MASTER_I => intercon.slaves.master(sl_flash_id),
    SLAVE_O => intercon.slaves.slave(sl_flash_id),

    IRQ_I => irqs,
    IRQ_O => irqos(irqdev_flash_id),

    flash_s_n_o => flash_s_n,
    flash_c_o => flash_c,
    flash_hold_n_o => flash_hold_n,
    flash_w_n_o => flash_w_n,
    flash_d_o => flash_d,
    flash_q_i => flash_q,

    debug => open
  );

  inst_wb_sl_bram : wb_sl_bram
  generic map
  (
    nr_of_irqiports => nr_of_irqs,
    nr_of_irqoports => nr_of_irqs,
    nr_of_dbgports => 1
  )
  port map
  (
    SYSCON_I => intercon.slaves.syscon(sl_bram_id),
    MASTER_I => intercon.slaves.master(sl_bram_id),
    SLAVE_O => intercon.slaves.slave(sl_bram_id),

    IRQ_I => irqs,
    IRQ_O => irqos(irqdev_bram_id),

    debug => open
  );
  
  inst_wb_sl_speedtest : wb_sl_speedtest
  generic map
  (
    nr_of_irqiports => nr_of_irqs,
    nr_of_irqoports => nr_of_irqs,
    nr_of_dbgports => 1
  )
  port map
  (
    SYSCON_I => intercon.slaves.syscon(sl_speedtest_id),
    MASTER_I => intercon.slaves.master(sl_speedtest_id),
    SLAVE_O => intercon.slaves.slave(sl_speedtest_id),

    IRQ_I => irqs,
    IRQ_O => irqos(irqdev_speedtest_id),

    debug => open
  );

  inst_wb_sl_gpio : wb_sl_gpio
  generic map
  (
    nr_of_irqiports => nr_of_irqs,
    nr_of_irqoports => nr_of_irqs,
    nr_of_dbgports => 1
  )
  port map
  (
    SYSCON_I => intercon.slaves.syscon(sl_gpio_id),
    MASTER_I => intercon.slaves.master(sl_gpio_id),
    SLAVE_O => intercon.slaves.slave(sl_gpio_id),

    IRQ_I => irqs,
    IRQ_O => irqos(irqdev_gpio_id),

    gpio_io => pin_gpio_io,

    debug => open
  );

end RTL;