--------------------------------------------------------------------------------
-- copyright notice:
--------------------------------------------------------------------------------
-- copyright (c) 2007 by
--
-- CESYS GmbH / GERMANY
--
-- http://www.cesys.com
--------------------------------------------------------------------------------
-- author:
--------------------------------------------------------------------------------
-- Stefan Fischer <stefan.fischer@cesys.com>
--------------------------------------------------------------------------------
-- license:
--------------------------------------------------------------------------------
-- THIS SOURCECODE IS NOT FREE! IT IS FOR USE TOGETHER WITH THE CESYS EFM01
-- USB CARD (ARTICLE-NR.: C1050-4107) ONLY! YOU ARE NOT ALLOWED TO MODIFY AND
-- DISTRIBUTE OR USE IT WITH ANY OTHER HARDWARE, SOFTWARE OR ANY OTHER KIND OF
-- ASIC OR PROGRAMMABLE LOGIC DESIGN WITHOUT THE EXPLICIT PERMISSION OF THE
-- COPYRIGHT HOLDER!
--------------------------------------------------------------------------------
-- disclaimer of warranty:
--------------------------------------------------------------------------------
-- THIS SOURCECODE IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
-- THERE IS NO WARRANTY OR SUPPORT FOR THIS SOURCECODE. THE COPYRIGHT HOLDER 
-- PROVIDES THIS SOURCECODE "AS IS" WITHOUT WARRANTY OF ANY KIND,
-- EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
-- WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
-- THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THIS SOURCECODE IS WITH
-- YOU. SHOULD THIS SOURCECODE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL 
-- NECESSARY SERVICING, REPAIR OR CORRECTION.
--
-- IN NO EVENT WILL THE COPYRIGHT HOLDER BE LIABLE TO YOU FOR DAMAGES, INCLUDING
-- ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
-- USE OR INABILITY TO USE THIS SOURCECODE (INCLUDING BUT NOT LIMITED TO LOSS OF
-- DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD 
-- PARTIES OR A FAILURE OF THIS SOURCECODE TO OPERATE WITH ANY OTHER
-- SOFTWARE-PROGRAMS, HARDWARE-CIRCUITS OR ANY OTHER KIND OF ASIC OR
-- PROGRAMMABLE LOGIC DESIGN), EVEN IF THE COPYRIGHT HOLDER HAS BEEN ADVISED OF 
-- THE POSSIBILITY OF SUCH DAMAGES.
--------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.all;

use work.wishbone.all;


entity wb_sl_gpio is
  generic
  (
    nr_of_irqiports : positive := 1;
    nr_of_irqoports : positive := 1;
    nr_of_dbgports : positive := 1
  );
  port
  (
    SYSCON_I : in rec_syscon_port;
    MASTER_I : in rec_master_port;
    SLAVE_O : out rec_slave_port;

    IRQ_I : in std_logic_vector((nr_of_irqiports-1) downto 0);
    IRQ_O : out std_logic_vector((nr_of_irqoports-1) downto 0);
    
    gpio_io : inout std_logic_vector(49 downto 0);

    debug : inout std_logic_vector((nr_of_dbgports-1) downto 0)
  );
end wb_sl_gpio;


architecture RTL of wb_sl_gpio is

  signal wishbone : rec_wishbone_signal := wishbone_default;

  signal irqi : std_logic_vector((nr_of_irqiports-1) downto 0) := (others => '0');
  signal irqo : std_logic_vector((nr_of_irqoports-1) downto 0) := (others => '0');

  constant INPUT : std_logic := '0';
  constant OUTPUT : std_logic := not INPUT;

  type rec_gpio is record
    oe : std_logic_vector(49 downto 0);
    output : std_logic_vector(49 downto 0);
    input : std_logic_vector(49 downto 0);
    input_buff : std_logic_vector(49 downto 0);
  end record;
  constant gpio_default : rec_gpio :=
  (
    oe => (others => INPUT),
    output => (others => '0'),
    input => (others => '0'),
    input_buff => (others => '0')
  );
  signal gpio : rec_gpio := gpio_default;

begin

  -- connect ports to internal signals
  ------------------------------------
  
  -- ports of WISHBONE bus
  wishbone.syscon <= SYSCON_I;
  wishbone.master <= MASTER_I;
  SLAVE_O <= wishbone.slave;

  -- ports of interrupt bus
  irqi <= IRQ_I;
  IRQ_O <= irqo;
  
  -- ports of gpio
  process(gpio)
  begin
    for i in 0 to 49 loop
      if gpio.oe(i) = OUTPUT then
        gpio_io(i) <= gpio.output(i);
      else
        gpio_io(i) <= 'Z';
      end if;
    end loop;
  end process;
  gpio.input <= gpio_io;
  
  debug <=
  (
    others => 'Z'
  );

  -- architecture implementation
  ------------------------------

  -- WISHBONE handshake and external 50-bit GPIO
  -- input-, output- and output-enable-registers
  process(wishbone, gpio)
  begin

    -- clocked process part
    -----------------------

    if wishbone.syscon.rst = '1' then
      wishbone.slave.ack <= '0';
      gpio.oe <= (others => INPUT);
    elsif wishbone.syscon.clk'event and wishbone.syscon.clk = '1' then
      -- WISHBONE handshake acknowledge response for registers
      if wishbone.slave.ack = '1' then
        wishbone.slave.ack <= '0';
      elsif (wishbone.master.cyc and wishbone.master.stb) = '1' then
        wishbone.slave.ack <= '1';
      end if;
      -- WISHBONE write address-decoding
      if (wishbone.master.cyc and wishbone.master.stb and wishbone.master.we) = '1' then
        case wishbone.master.adr(rng_gpio_adr) is
          -- output registers
          when gpio_0_offset(rng_gpio_adr) =>
            gpio.output(31 downto 0) <= wishbone.master.dat;
          when gpio_1_offset(rng_gpio_adr) =>
            gpio.output(49 downto 32) <= wishbone.master.dat(rng_gpio_1);
          -- output-enable registers
          when gpio_0_oe_offset(rng_gpio_adr) =>
            gpio.oe(31 downto 0) <= wishbone.master.dat;
          when gpio_1_oe_offset(rng_gpio_adr) =>
            gpio.oe(49 downto 32) <= wishbone.master.dat(rng_gpio_1);
          when others => null;
        end case;
      end if;
      -- route GPIO inputs through flip-flops
      gpio.input_buff <= gpio.input;
    end if;

    -- combinatorial process part
    -----------------------------

    -- WISHBONE read address-decoding
    wishbone.slave.dat <= (others => '0'); -- default value
    case wishbone.master.adr(rng_gpio_adr) is
      -- input registers
      when gpio_0_offset(rng_gpio_adr) =>
        wishbone.slave.dat <= gpio.input_buff(31 downto 0);
      when gpio_1_offset(rng_gpio_adr) =>
        wishbone.slave.dat(rng_gpio_1) <= gpio.input_buff(49 downto 32);
      -- output-enable registers
      when gpio_0_oe_offset(rng_gpio_adr) =>
        wishbone.slave.dat <= gpio.oe(31 downto 0);
      when gpio_1_oe_offset(rng_gpio_adr) =>
        wishbone.slave.dat(rng_gpio_1) <= gpio.oe(49 downto 32);
      when others => null;
    end case;

  end process;
  
  -- embedded comma separated value file describing relationship between logical and physical i/o name
  -- 1. create a new text file with "*.csv" extension
  -- 2. copy and paste pinout information (between "pragma CSV_ON/OFF")
  -- 3. remove vhdl comments "--"
  -- 4. import to an appropriate spreadsheet programm
  -- pragma CSV_ON
  -- con,net,hdl,comment
  -- ,,,
  -- J3/2,EXT1_IO0,gpio[0],
  -- J3/3,EXT1_IO1,gpio[1],
  -- J3/4,EXT1_IO2,gpio[2],
  -- J3/5,EXT1_IO3,gpio[3],
  -- J3/6,EXT1_IO4,gpio[4],
  -- J3/9,EXT1_IO5,gpio[5],
  -- J3/10,EXT1_IO6,gpio[6],
  -- J3/11,EXT1_IO7,gpio[7],
  -- J3/13,EXT1_IO8,gpio[8],
  -- J3/14,EXT1_IO9,gpio[9],
  -- J3/15,EXT1_IO10,gpio[10],
  -- J3/16,EXT1_IO11,gpio[11],
  -- J3/17,EXT1_IO12,gpio[12],
  -- J3/18,EXT1_IO13,gpio[13],
  -- J3/19,EXT1_IO14,gpio[14],
  -- J3/20,EXT1_IO15,gpio[15],
  -- J3/22,EXT1_IO16,gpio[16],
  -- J3/23,EXT1_IO17,gpio[17],
  -- J3/25,EXT1_IO19,gpio[18],
  -- J3/27,EXT1_IO20,gpio[19],
  -- J3/28,EXT1_IO21,gpio[20],
  -- J3/29,EXT1_IO22,gpio[21],
  -- J3/30,EXT1_IO23,gpio[22],
  -- J3/31,EXT1_IO24,gpio[23],
  -- J3/32,EXT1_IO25,gpio[24],
  -- J3/34,EXT1_IO26,gpio[25],
  -- ,,,
  -- J4/2,EXT2_IO0,gpio[26],
  -- J4/3,EXT2_IO1,gpio[27],
  -- J4/4,EXT2_IO2,gpio[28],
  -- J4/6,EXT2_IO4,gpio[29],
  -- J4/7,EXT2_IO5,gpio[30],led
  -- J4/9,EXT2_IO6,gpio[31],
  -- J4/10,EXT2_IO7,gpio[32],
  -- J4/11,EXT2_IO8,gpio[33],
  -- J4/12,EXT2_IO9,gpio[34],
  -- J4/14,EXT2_IO10,gpio[35],
  -- J4/15,EXT2_IO11,gpio[36],
  -- J4/20,EXT2_IO13,gpio[37],
  -- J4/21,EXT2_IO14,gpio[38],
  -- J4/22,EXT2_IO15,gpio[39],
  -- J4/23,EXT2_IO16,gpio[40],
  -- J4/24,EXT2_IO17,gpio[41],
  -- J4/25,EXT2_IO18,gpio[42],
  -- J4/27,EXT2_IO19,gpio[43],
  -- J4/28,EXT2_IO20,gpio[44],
  -- J4/29,EXT2_IO21,gpio[45],
  -- J4/30,EXT2_IO22,gpio[46],
  -- J4/31,EXT2_IO23,gpio[47],
  -- J4/32,EXT2_IO24,gpio[48],
  -- J4/34,EXT2_IO25,gpio[49],
  -- pragma CSV_OFF

end RTL;