--------------------------------------------------------------------------------
-- copyright notice:
--------------------------------------------------------------------------------
-- copyright (c) 2007 by
--
-- CESYS GmbH / GERMANY
--
-- http://www.cesys.com
--------------------------------------------------------------------------------
-- author:
--------------------------------------------------------------------------------
-- Stefan Fischer <stefan.fischer@cesys.com>
--------------------------------------------------------------------------------
-- license:
--------------------------------------------------------------------------------
-- THIS SOURCECODE IS NOT FREE! IT IS FOR USE TOGETHER WITH THE CESYS EFM01
-- USB CARD (ARTICLE-NR.: C1050-4107) ONLY! YOU ARE NOT ALLOWED TO MODIFY AND
-- DISTRIBUTE OR USE IT WITH ANY OTHER HARDWARE, SOFTWARE OR ANY OTHER KIND OF
-- ASIC OR PROGRAMMABLE LOGIC DESIGN WITHOUT THE EXPLICIT PERMISSION OF THE
-- COPYRIGHT HOLDER!
--------------------------------------------------------------------------------
-- disclaimer of warranty:
--------------------------------------------------------------------------------
-- THIS SOURCECODE IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
-- THERE IS NO WARRANTY OR SUPPORT FOR THIS SOURCECODE. THE COPYRIGHT HOLDER 
-- PROVIDES THIS SOURCECODE "AS IS" WITHOUT WARRANTY OF ANY KIND,
-- EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
-- WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
-- THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THIS SOURCECODE IS WITH
-- YOU. SHOULD THIS SOURCECODE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL 
-- NECESSARY SERVICING, REPAIR OR CORRECTION.
--
-- IN NO EVENT WILL THE COPYRIGHT HOLDER BE LIABLE TO YOU FOR DAMAGES, INCLUDING
-- ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
-- USE OR INABILITY TO USE THIS SOURCECODE (INCLUDING BUT NOT LIMITED TO LOSS OF
-- DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD 
-- PARTIES OR A FAILURE OF THIS SOURCECODE TO OPERATE WITH ANY OTHER
-- SOFTWARE-PROGRAMS, HARDWARE-CIRCUITS OR ANY OTHER KIND OF ASIC OR
-- PROGRAMMABLE LOGIC DESIGN), EVEN IF THE COPYRIGHT HOLDER HAS BEEN ADVISED OF 
-- THE POSSIBILITY OF SUCH DAMAGES.
--------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.NUMERIC_STD.all;


package wishbone is

  -- basic system configuration
  -----------------------------

  -- nr of devices
  constant nr_of_masters : positive := 1;
  subtype rng_masters is natural range (nr_of_masters-1) downto 0;
  constant nr_of_slaves : positive := 4;
  subtype rng_slaves is natural range (nr_of_slaves-1) downto 0;

  -- nr of address- and data-lines
  constant adr_width : positive := 32;
  subtype rng_adr is natural range (adr_width-1) downto 0;
  constant dat_width : positive := 32;
  subtype rng_dat is natural range (dat_width-1) downto 0;
  
  -- interrupt settings
  ---------------------
  
  -- nr of modules connected to the interrupt bus
  constant nr_of_irqdevs : positive := 5;
  subtype rng_irqdevs is natural range (nr_of_irqdevs-1) downto 0;
  -- nr of global IRQs
  constant nr_of_irqs : positive := 1;
  subtype rng_irqs is natural range (nr_of_irqs-1) downto 0;
  -- signals building the interrupt bus
  type arr_irqos is array(natural range <>) of std_logic_vector(rng_irqs);
  -- each module connected to the interrupt bus gets an unique interrupt device id
  signal irqos : arr_irqos(rng_irqdevs) := (others => (others => '0'));
  -- each IRQ on the interrupt bus gets an unique interrupt id
  signal irqs : std_logic_vector(rng_irqs) := (others => '0');

  -- WISHBONE specific data types and default values
  --------------------------------------------------

  -- ports driven by WISHBONE syscon modules
  type rec_syscon_port is record
    rst : std_logic;
    clk : std_logic;
  end record;
  constant syscon_default : rec_syscon_port :=
  (
    rst => '1',
    clk => '0'
  );
  type arr_syscon_port is array(natural range <>) of rec_syscon_port;

  -- ports driven by WISHBONE master modules
  type rec_master_port is record
    cyc : std_logic;
    stb : std_logic;
    we : std_logic;
    adr : std_logic_vector(rng_adr);
    dat : std_logic_vector(rng_dat);
  end record;
  constant master_default : rec_master_port :=
  (
    cyc => '0',
    stb => '0',
    we => '0',
    adr => (others => '0'),
    dat => (others => '0')
  );
  type arr_master_port is array(natural range <>) of rec_master_port;

  -- ports driven by WISHBONE slave modules
  type rec_slave_port is record
    ack : std_logic;
    dat : std_logic_vector(rng_dat);
  end record;
  constant slave_default : rec_slave_port :=
  (
    ack => '0',
    dat => (others => '0')
  );
  type arr_slave_port is array(natural range <>) of rec_slave_port;

  -- the whole WISHBONE signal
  type rec_wishbone_signal is record
    syscon : rec_syscon_port;
    master : rec_master_port;
    slave : rec_slave_port;
  end record;
  constant wishbone_default : rec_wishbone_signal :=
  (
    syscon => syscon_default,
    master => master_default,
    slave => slave_default
  );

  -- signals connecting masters to intercon module
  type rec_masters_signal is record
    syscon : arr_syscon_port(rng_masters);
    master : arr_master_port(rng_masters);
    slave : arr_slave_port(rng_masters);
  end record;
  constant masters_default : rec_masters_signal :=
  (
    syscon => (others => syscon_default),
    master => (others => master_default),
    slave => (others => slave_default)
  );

  -- signals connecting slaves to intercon module
  type rec_slaves_signal is record
    syscon : arr_syscon_port(rng_slaves);
    master : arr_master_port(rng_slaves);
    slave : arr_slave_port(rng_slaves);
  end record;
  constant slaves_default : rec_slaves_signal :=
  (
    syscon => (others => syscon_default),
    master => (others => master_default),
    slave => (others => slave_default)
  );

  -- all signals connected to intercon module
  type rec_intercon_signal is record
    syscon : rec_syscon_port;
    masters : rec_masters_signal;
    slaves : rec_slaves_signal;
  end record;
  constant intercon_default : rec_intercon_signal :=
  (
    syscon => syscon_default,
    masters => masters_default,
    slaves => slaves_default
  );
  
  -- WISHBONE modules
  -------------------

  -- interconnection logic
  component wb_intercon is
    generic
    (
      nr_of_dbgports : positive := 1
    );
    port
    (
      SYSCON_I : in rec_syscon_port;

      SYSCON_MA_O : out arr_syscon_port(rng_masters);
      MASTER_I : in arr_master_port(rng_masters);
      SLAVE_O : out arr_slave_port(rng_masters);

      SYSCON_SL_O : out arr_syscon_port(rng_slaves);
      MASTER_O : out arr_master_port(rng_slaves);
      SLAVE_I : in arr_slave_port(rng_slaves);

      debug : inout std_logic_vector((nr_of_dbgports-1) downto 0)
    );
  end component;

  -- a single signal connects all WISHBONE devices to intercon
  signal intercon : rec_intercon_signal := intercon_default;

  -- partial address decoding is done in the WISHBONE system
  -- some msbs of the address are used to select the appropriate slave device
  -- in the interconnection logic (base address decoding)
  -- an unique slave id is derived from the unsigned integer interpretation of these bits
  -- the slave id is used as an index and identifies the slave module in array data types
  -- the master id is assigned manually
  -- please note, that the software uses byte addressing, while the WISHBONE
  -- system uses 32-bit word addressing!
  -- => WISHBONE address := software address rightshift by 2
  -- some lsbs of the address are used to select which memory unit or parameter to read
  -- or write inside slave devices (address offset decoding)
  -- FPGA resources can be saved by decoding only the necessary bits
  -- example: if you have five parameters in your slave device,
  -- then you decode only the three lsbs
  -- absolute address := base address + offset
  subtype rng_slave_select is natural range 19 downto 18;

  -- module providing global system signals clock and reset
  ---------------------------------------------------------

  component wb_syscon is
    generic
    (
      nr_of_dbgports : positive := 1
    );
    port
    (
      SYSCON_O : out rec_syscon_port;
      
      sys_rst_i : in std_logic;
      sys_clk_i : in std_logic;

      debug : inout std_logic_vector((nr_of_dbgports-1) downto 0)
    );
  end component;

  signal sys_rst : std_logic := '1';
  signal sys_clk : std_logic := '0';

  -- fx2 slave fifo interface <=> WISHBONE bridge
  -----------------------------------------------

  constant ma_fx2_id : natural := 0;
  component wb_ma_fx2 is
    generic
    (
      nr_of_irqiports : positive := 1;
      nr_of_irqoports : positive := 1;
      nr_of_dbgports : positive := 1
    );
    port
    (
      SYSCON_I : in rec_syscon_port;
      MASTER_O : out rec_master_port;
      SLAVE_I : in rec_slave_port;

      IRQ_I : in std_logic_vector((nr_of_irqiports-1) downto 0);
      IRQ_O : out std_logic_vector((nr_of_irqoports-1) downto 0);

      fx2_ifclk_i : in std_logic;
      fx2_flag_n_i : in std_logic_vector(2 downto 0);
      fx2_sloe_n_o : out std_logic;
      fx2_slrd_n_o : out std_logic;
      fx2_slwr_n_o : out std_logic;
      fx2_pktend_n_o : out std_logic;
      fx2_fifoadr_o : out std_logic_vector(1 downto 0);
      fx2_fd_io : inout std_logic_vector(15 downto 0);
  
      debug : inout std_logic_vector((nr_of_dbgports-1) downto 0)
    );
  end component;
  constant irqdev_fx2_id : natural := 0;

  -- bram controller
  ------------------
  
  -- define bram memory space
  constant bram_baseadr : std_logic_vector(rng_adr) := x"0000_0000";
  constant bram_highadr : std_logic_vector(rng_adr) := x"0000_01FF";
  constant bram_alignment : std_logic_vector(rng_adr) := x"0000_0001";
  -- valid address bits
  subtype rng_bram_adr is natural range 8 downto 0;

  -- pragma SW_ON
  --#define BRAM_BASEADR 0x00000000
  --#define BRAM_HIGHADR 0x000007FF
  --#define BRAM_ALIGNMENT 0x00000004
  -- pragma SW_OFF

  constant sl_bram_id : natural := TO_INTEGER(unsigned(bram_baseadr(rng_slave_select)));
  component wb_sl_bram is
    generic
    (
      nr_of_irqoports : positive := 1;
      nr_of_irqiports : positive := 1;
      nr_of_dbgports : positive := 1
    );
    port
    (
      SYSCON_I : in rec_syscon_port;
      MASTER_I : in rec_master_port;
      SLAVE_O : out rec_slave_port;

      IRQ_I : in std_logic_vector((nr_of_irqiports-1) downto 0);
      IRQ_O : out std_logic_vector((nr_of_irqoports-1) downto 0);

      debug : inout std_logic_vector((nr_of_dbgports-1) downto 0)
    );
  end component;
  constant irqdev_bram_id : natural := 1;

  -- GPIO
  -------

  -- define GPIO base address
  constant gpio_baseadr : std_logic_vector(rng_adr) := x"0004_0000";
  constant gpio_alignment : std_logic_vector(rng_adr) := x"0000_0001";
  -- valid address bits
  subtype rng_gpio_adr is natural range 1 downto 0;
  -- I/Os
  constant gpio_0_offset : std_logic_vector(rng_adr) := x"0000_0000";
  constant gpio_1_offset : std_logic_vector(rng_adr) := x"0000_0001";
  -- I/Os output-enable
  constant gpio_0_oe_offset : std_logic_vector(rng_adr) := x"0000_0002";
  constant gpio_1_oe_offset : std_logic_vector(rng_adr) := x"0000_0003";
  -- define bitmask for external I/Os and I/Os output-enable
  subtype rng_gpio_0 is natural range 31 downto 0;
  subtype rng_gpio_1 is natural range 17 downto 0;

  -- pragma SW_ON
  --#define GPIO_BASEADR 0x00100000
  --#define GPIO_ALIGNMENT 0x00000004
  --#define GPIO_0_OFFSET 0x00000000
  --#define GPIO_1_OFFSET 0x00000004
  --#define GPIO_0_OE_OFFSET 0x00000008
  --#define GPIO_1_OE_OFFSET 0x0000000C
  --#define GPIO_0_MASK 0xFFFFFFFF
  --#define GPIO_1_MASK 0x0003FFFF
  -- pragma SW_OFF

  constant sl_gpio_id : natural := TO_INTEGER(unsigned(gpio_baseadr(rng_slave_select)));
  component wb_sl_gpio is
    generic
    (
      nr_of_irqiports : positive := 1;
      nr_of_irqoports : positive := 1;
      nr_of_dbgports : positive := 1
    );
    port
    (
      SYSCON_I : in rec_syscon_port;
      MASTER_I : in rec_master_port;
      SLAVE_O : out rec_slave_port;
  
      IRQ_I : in std_logic_vector((nr_of_irqiports-1) downto 0);
      IRQ_O : out std_logic_vector((nr_of_irqoports-1) downto 0);
      
      gpio_io : inout std_logic_vector(49 downto 0);
  
      debug : inout std_logic_vector((nr_of_dbgports-1) downto 0)
    );
  end component;
  constant irqdev_gpio_id : natural := 3;

  -- SPI flash controller
  -----------------------

  -- define flash memory space
  constant flash_baseadr : std_logic_vector(rng_adr) := x"0008_0000";
  constant flash_highadr : std_logic_vector(rng_adr) := x"0009_FFFF";
  constant flash_alignment : std_logic_vector(rng_adr) := x"0000_0001";
  -- valid address bits
  subtype rng_flash_adr is natural range 16 downto 0;
  -- define address of command register (write only!)
  constant flash_cmd_offset : std_logic_vector(rng_adr) := x"0002_0000";
  -- define bitmask for command register
  subtype rng_flash_cmd is natural range 0 downto 0;
  -- define erase command
  constant flash_cmd_val_erase : std_logic_vector(rng_dat) := x"0000_0000";

  -- pragma SW_ON
  --#define FLASH_BASEADR 0x00200000
  --#define FLASH_HIGHADR 0x0027FFFF
  --#define FLASH_ALIGNMENT 0x00000004
  --#define FLASH_CMD_OFFSET 0x00080000 /* write only ! */
  --#define FLASH_CMD_MASK 0x00000001
  --#define FLASH_CMD_VAL_ERASE 0x00000000
  -- pragma SW_OFF

  constant sl_flash_id : natural := TO_INTEGER(unsigned(flash_baseadr(rng_slave_select)));
  component wb_sl_flash is
    generic
    (
      nr_of_irqoports : positive := 1;
      nr_of_irqiports : positive := 1;
      nr_of_dbgports : positive := 1
    );
    port
    (
      SYSCON_I : in rec_syscon_port;
      MASTER_I : in rec_master_port;
      SLAVE_O : out rec_slave_port;
  
      IRQ_I : in std_logic_vector((nr_of_irqiports-1) downto 0);
      IRQ_O : out std_logic_vector((nr_of_irqoports-1) downto 0);
  
      flash_s_n_o : out std_logic;
      flash_c_o : out std_logic;
      flash_hold_n_o : out std_logic;
      flash_w_n_o : out std_logic;
      flash_d_o : out std_logic;
      flash_q_i : in std_logic;
  
      debug : inout std_logic_vector((nr_of_dbgports-1) downto 0)
    );
  end component;
  constant irqdev_flash_id : natural := 2;

  signal flash_hold_n : std_logic := '1';
  signal flash_w_n : std_logic := '1';
  
  -- speedtest register
  ---------------------
  
  -- define speedtest memory space
  constant speedtest_baseadr : std_logic_vector(rng_adr) := x"000C_0000";
  constant speedtest_highadr : std_logic_vector(rng_adr) := x"000C_0000";
  constant speedtest_alignment : std_logic_vector(rng_adr) := x"0000_0001";

  -- pragma SW_ON
  --#define SPEEDTEST_BASEADR 0x00300000
  --#define SPEEDTEST_HIGHADR 0x00300000
  --#define SPEEDTEST_ALIGNMENT 0x00000004
  -- pragma SW_OFF

  constant sl_speedtest_id : natural := TO_INTEGER(unsigned(speedtest_baseadr(rng_slave_select)));
  component wb_sl_speedtest is
    generic
    (
      nr_of_irqoports : positive := 1;
      nr_of_irqiports : positive := 1;
      nr_of_dbgports : positive := 1
    );
    port
    (
      SYSCON_I : in rec_syscon_port;
      MASTER_I : in rec_master_port;
      SLAVE_O : out rec_slave_port;

      IRQ_I : in std_logic_vector((nr_of_irqiports-1) downto 0);
      IRQ_O : out std_logic_vector((nr_of_irqoports-1) downto 0);

      debug : inout std_logic_vector((nr_of_dbgports-1) downto 0)
    );
  end component;
  constant irqdev_speedtest_id : natural := 4;

end wishbone;